/*
 * adxl345.h
 *
 *  Created on: Nov 7, 2024
 *      Author: root
 */

#ifndef INC_ADXL345_H_
#define INC_ADXL345_H_


#include "stdint.h"


// ADXL345 Register Map
#define ADXL345_DEVID             (uint8_t)0x00  // Device ID
#define ADXL345_THRESH_TAP        (uint8_t)0x1D  // Tap threshold
#define ADXL345_OFSX              (uint8_t)0x1E  // X-axis offset
#define ADXL345_OFSY              (uint8_t)0x1F  // Y-axis offset
#define ADXL345_OFSZ              (uint8_t)0x20  // Z-axis offset
#define ADXL345_DUR               (uint8_t)0x21  // Tap duration
#define ADXL345_LATENT            (uint8_t)0x22  // Tap latency
#define ADXL345_WINDOW            (uint8_t)0x23  // Tap window
#define ADXL345_THRESH_ACT        (uint8_t)0x24  // Activity threshold
#define ADXL345_THRESH_INACT      (uint8_t)0x25  // Inactivity threshold
#define ADXL345_TIME_INACT        (uint8_t)0x26  // Inactivity time
#define ADXL345_ACT_INACT_CTL     (uint8_t)0x27  // Axis enable control for activity and inactivity detection
#define ADXL345_THRESH_FF         (uint8_t)0x28  // Free-fall threshold
#define ADXL345_TIME_FF           (uint8_t)0x29  // Free-fall time
#define ADXL345_TAP_AXES          (uint8_t)0x2A  // Axis control for single/double tap
#define ADXL345_ACT_TAP_STATUS    (uint8_t)0x2B  // Source of single/double tap
#define ADXL345_BW_RATE           (uint8_t)0x2C  // Data rate and power mode control
#define ADXL345_POWER_CTL         (uint8_t)0x2D  // Power-saving features control
#define ADXL345_INT_ENABLE        (uint8_t)0x2E  // Interrupt enable control
#define ADXL345_INT_MAP           (uint8_t)0x2F  // Interrupt mapping control
#define ADXL345_INT_SOURCE        (uint8_t)0x30  // Source of interrupts
#define ADXL345_DATA_FORMAT       (uint8_t)0x31  // Data format control
#define ADXL345_DATAX0            (uint8_t)0x32  // X-Axis Data 0
#define ADXL345_DATAX1            (uint8_t)0x33  // X-Axis Data 1
#define ADXL345_DATAY0            (uint8_t)0x34  // Y-Axis Data 0
#define ADXL345_DATAY1            (uint8_t)0x35  // Y-Axis Data 1
#define ADXL345_DATAZ0            (uint8_t)0x36  // Z-Axis Data 0
#define ADXL345_DATAZ1            (uint8_t)0x37  // Z-Axis Data 1
#define ADXL345_FIFO_CTL          (uint8_t)0x38  // FIFO control
#define ADXL345_FIFO_STATUS       (uint8_t)0x39  // FIFO status


#define PWR_RST					  (uint8_t)0x00 //Reset power register

#define MEASUREMODE				  (uint8_t)0x08 //Set ADXL345 into measure mode

typedef struct
{
	float ax;
	float ay;
	float az;
}ADXL345_AccData;


typedef struct
{
	uint8_t Accel_Range;
	uint8_t InterruptEnable;
	uint8_t DataReadyInterrupt;
	/*rest of the interrupt shall be implement by the user*/
}ADXL345_InitParam;

typedef enum
{
	accl_2g=0,
	accl_4g=1,
	accl_8g=2,
	accl_16g=3
}adxl345Parameters;


typedef enum
{
	Interrupt_Disabled=0,
	Interrupt_Enabled=1
}adxl345InterruptsEnable;


typedef enum
{

	Int_DataReady=(1<<7),
	Int_SingleTap=(1<<6),
	Int_DoubleTap=(1<<5),
	Int_Activity=(1<<4),
	Int_Inactivity=(1<<3),
	Int_FreeFall=(1<<2),
	Int_Watermark=(1<<1),
	Int_Overrun=(1<<0)
}adxl345InterruptsSource;




void adxl345_SetAdd(uint8_t add);

uint8_t adxl345_ReadID(void);

void adxl345_Init(ADXL345_InitParam *Param);

void adxl345_update(ADXL345_AccData *accData);


adxl345InterruptsSource adxl345_ReadInterrupt(void);


#endif /* INC_ADXL345_H_ */
