/*
 * adxl345.c
 *
 *  Created on: Nov 7, 2024
 *      Author: root
 */

#include "adxl345.h"

#include "i2c.h"


uint8_t ADXL345_Add = (0x53<<1);


uint8_t acc_range=0;



static void adxl345_write(uint8_t reg, uint8_t *data, uint8_t len)
{
	HAL_I2C_Mem_Write(&hi2c1, ADXL345_Add, reg, 1, data, len, 100);
}


static void adxl345_read(uint8_t reg, uint8_t *data, uint8_t len)
{
	HAL_I2C_Mem_Read(&hi2c1, ADXL345_Add, reg, 1, data, len, 100);
}


void adxl345_SetAdd(uint8_t add)
{
	ADXL345_Add=(add<<1);
}


uint8_t adxl345_ReadID(void)
{
	uint8_t ret_data;
	adxl345_read(ADXL345_DEVID, &ret_data, 1);
	return ret_data;
}

void adxl345_Init(ADXL345_InitParam *Param)
{
	/*Get the acceleration range */
	acc_range=Param->Accel_Range;

	uint8_t temp[1];

	/*Set the acceleration range*/
	temp[0]=acc_range;
	adxl345_write(ADXL345_DATA_FORMAT, temp, 1);

	/*Reset Power Control register*/
	temp[0]=PWR_RST;
	adxl345_write(ADXL345_POWER_CTL, temp, 1);

	/*Set ADXL345 into measure mode*/
	temp[0]=MEASUREMODE;
	adxl345_write(ADXL345_POWER_CTL, temp, 1);

	if(Param->InterruptEnable==Interrupt_Enabled)
	{
		temp[0]=0x00;
		if(Param->DataReadyInterrupt==Int_DataReady)
		{
			temp[0]|=Int_DataReady;

		}
		/*The rest shall be implement by the user
		 * temp[0]|=Int_SingleTap
		 * */
		adxl345_write(ADXL345_INT_ENABLE,temp,1);

		temp[0]=0;
		adxl345_read(ADXL345_INT_ENABLE,temp,1);

	}



}

void adxl345_update(ADXL345_AccData *accData)
{
	uint8_t adxl345Data[6];

	adxl345_read(ADXL345_DATAX0,adxl345Data,6);

	int16_t x,y,z;

	x = ((adxl345Data[1]<<8)|adxl345Data[0]);
	y = ((adxl345Data[3]<<8)|adxl345Data[2]);
	z = ((adxl345Data[5]<<8)|adxl345Data[4]);

	float divider;
	switch(acc_range)
	{
		case accl_2g: divider=0.003906; /*1/256*/	break;
		case accl_4g: divider=0.0078125;/*1/128*/	break;
		case accl_8g: divider=0.01563;	/*1/64*/	break;
		case accl_16g: divider=0.03125;	/*1/32*/	break;
	}

	accData->ax=x*divider;
	accData->ay=y*divider;
	accData->az=z*divider;
}


adxl345InterruptsSource adxl345_ReadInterrupt(void)
{
	uint8_t temp[1];
	adxl345_read(ADXL345_INT_SOURCE,temp,1);

	if((temp[0]&Int_DataReady)==Int_DataReady){return Int_DataReady;}

	/*rest shall be implement by the user*/
	return 0;
}
